#ifndef CLASS_ELEMENT
#define CLASS_ELEMENT

#include <vector>
#include <map>

#include "graphic/Image.h"
#include "IsoObject.h"
#include "StatisticalValue.h"

#define NUM_VIEWS 8

/* views */
#define NORTH 0
#define NORTH_EAST 1
#define EAST 2
#define SOUTH_EAST 3
#define SOUTH 4
#define SOUTH_WEST 5
#define WEST 6
#define NORTH_WEST 7
#define NO_DIRECTION 8

#define SELF -1
#define ANY 8

/* types of element */
#define SCENE_ELEMENT 0
#define BUILDING 1
#define UNIT 2
#define TERRAIN_ELEMENT 3

/// A map containing StatisticalValues
typedef std::map<int, StatisticalValue * > Statsmap;

/// A base class for elements that can be present on the landscape
/// DO NOT USE this class for create objects, USE its subclasses (Unit, SceneElement, Building) 
class Element : public IsoObject
{
	protected:
	 /// images of the element
	 std::vector < graphic::Image * > _views;
	 /// statiticals data
	 Statsmap _stats;
	 /// type of the element
	 int _type;
	 /// direction of the element
	 int _direction;

	public:
	 /// Base constructor
	 Element () { };
	 /// Constructor
	 Element(int r, int c, Statsmap stats);
	 /// Full constructor
	 Element(int r, int c, std::vector < graphic::Image * > views, Statsmap stats);
	 /// Destroyer
	 virtual ~Element();

	 /// set the image that should be blitted
	 virtual void SetDirection(int direction)
	 {
		_direction = direction;
		if(direction != NO_DIRECTION)
			_img = _views[direction];
		else
			_img = _views[SOUTH];
	 };

	 /// return direction value
	 int GetDirection() { return _direction; };

	 /* STATS MANAGEMENT */
	 /// return the max of a statistical value
	 int GetMaxStat(int id) { return _stats[id]->GetMax(); }
	 /// set the max of a statistical value
	 void SetMaxStat(int id, int val) { _stats[id]->SetMax(val); };
	 /// add a value to the max statistical value
	 void AddMaxStat(int id, int val) { _stats[id]->AddMax(val); };
	 /// subtract a value from the max statistical value
	 void SubMaxStat(int id, int val) { _stats[id]->SubMax(val); };
	 /// return the current  value of a statistical value
	 int GetCurStat(int id) { return _stats[id]->GetCur(); }
	 /// set the current value of a statistical value
	 void SetCurStat(int id, int val) { _stats[id]->SetCur(val); };
	 /// add a value to the current statistical value
	 void AddCurStat(int id, int val) { _stats[id]->AddCur(val); };
	 /// subtract a value from the current statistical value
	 void SubCurStat(int id, int val) { _stats[id]->SubCur(val); };
	 /// restore the current value to the max
	 void RestoreCurStat(int id) { _stats[id]->RestoreCur(); };

	 /// return the type of the element
	 int GetType() { return _type;};

	 /// flip image and direction
	 void Flip()
	 {
		// flip direction
		_direction = (_direction + (NUM_VIEWS / 2)) % NUM_VIEWS;
		// set view
		_img = _views[_direction];
	 };

	 graphic::Image * GetImg(int ind) { return _views[ind]; };
};

#endif
