#include "PlayerContainer.h"

using namespace std;
using namespace graphic;

//Loads xml files and creates elements (unit/buildings and sceneelements)
bool PlayerContainer::CreateElements(const char *xmlpath)
{
	// Create a XmlLoader that reads the xml data
	XmlLoader load_data(xmlpath);
	
	do
	{
		if(load_data.GetType() == UNIT)
		{
			Unit *a_element;
			// Unit deleted in PlayerContainer destructor
			a_element = new Unit(load_data.GetRowSize(), load_data.GetColSize(), load_data.GetImages(), load_data.GetStats(_id));
			//Add new unit to objectmap at location point
			_objectmap_pointer->AddObject(a_element, load_data.GetRowPos(), load_data.GetColPos());
			// Gets Faction from xml file, if no faction tag is found NO_FACTION is used
			a_element->SetFaction(load_data.GetFaction());
			// Gets Direction from xml file, if no direction tag is found Unit is faced towards the center of the map
			a_element->SetDirection(load_data.GetDirection());
			if( a_element->GetDirection() == NO_DIRECTION)
				a_element->SetDirection(DirectionTowardsCenter(a_element));
			// Add unit to PlayerCointainers Unit storage
			_element_storage.push_back(a_element);
		}
		else if(load_data.GetType() == BUILDING)
		{
			Building *a_element;
			// Building deleted in PlayerContainer destructor
			a_element = new Building(load_data.GetRowSize(), load_data.GetColSize(), load_data.GetImages(), load_data.GetStats(_id));
			//Add new unit to objectmap at location point
			_objectmap_pointer->AddObject(a_element, load_data.GetRowPos(), load_data.GetColPos());
			// Gets Faction from xml file, if no faction tag is found NO_FACTION is used
			a_element->SetFaction(load_data.GetFaction());
			// Add unit to PlayerCointainers Unit storage
			_element_storage.push_back(a_element);
		}
		else if(load_data.GetType() == SCENE_ELEMENT)
		{
			SceneElement *a_element;
			a_element = new SceneElement(load_data.GetRowSize(), load_data.GetColSize(), load_data.GetImages(), load_data.GetStats(_id));
			//Add new unit to objectmap at location point
			_objectmap_pointer->AddObject(a_element, load_data.GetRowPos(), load_data.GetColPos());
			// Add unit to PlayerCointainers Unit storage
			_element_storage.push_back(a_element);	
		}
		else if(load_data.GetType() == TERRAIN_ELEMENT)
		{
			TerrainElement *a_element;
			// Unit deleted in PlayerContainer destructor
			a_element = new TerrainElement(load_data.GetRowSize(), load_data.GetColSize(), load_data.GetImages(), load_data.GetStats(_id));
			//Add new unit to objectmap at location point
			_objectmap_pointer->AddObject(a_element, load_data.GetRowPos(), load_data.GetColPos());
			// Gets Direction from xml file, if no direction tag is found Unit is faced towards the center of the map
			a_element->SetDirection(load_data.GetDirection());
			if( a_element->GetDirection() == NO_DIRECTION)
				a_element->SetDirection(DirectionTowardsCenter(a_element));
			// Add unit to PlayerCointainers Unit storage
			_element_storage.push_back(a_element);
		}
		_id++;
		
	}while(load_data.NextElement()); //Keep reading while there are more xml tags
		
	return true;
}

// Places containers units in a group at row,col
void PlayerContainer::PlaceUnitGroup(int row, int col)
{
	int x = row;
	
	for( std::vector<Element *>::iterator it = _element_storage.begin(); it != _element_storage.end(); it++ )
	{
		if((*it)->GetType() == UNIT)
		{
			// Move unit until it is at destination
			_objectmap_pointer->MoveObject((*it), x, col);
			(*it)->SetPosition(_objectmap_pointer->GetObjX((*it)), _objectmap_pointer->GetObjY((*it)));
			(*it)->SetDirection(DirectionTowardsCenter(*it));
			x++;
		}
	}
}

// Calculates the visibilty for the objectmap with containers Elements
void PlayerContainer::ComputeVisibility()
{
	// reset TilesMap visibility
	_objectmap_pointer->ResetMapVisibility();

	for(std::vector<Element *>::iterator it = _element_storage.begin(); it != _element_storage.end(); it++)
	{
		if((*it)->GetType() == UNIT)
			_objectmap_pointer->ComputeObjVisibility((*it), (*it)->GetCurStat(STAT_VISIBILITY), (*it)->GetDirection());
		else if((*it)->GetType() == BUILDING)
			_objectmap_pointer->ComputeObjVisibility((*it), (*it)->GetCurStat(STAT_VISIBILITY));		
	}	
}

// Find and delete a element at row,col if it belongs to playercontainer
void PlayerContainer::DeleteElement(int row, int col)
{
	Element * aElement = (Element *)_objectmap_pointer->GetObject(row, col);
	if(aElement)
		DeleteElement(aElement->GetCurStat(STAT_ID));
}

// Delete selected element
void PlayerContainer::DeleteElement(Element * element)
{
	if(element)
		DeleteElement(element->GetCurStat(STAT_ID));
}

// Find and delete a element with ID if it belongs to playercontainer
void PlayerContainer::DeleteElement(int id)
{
	std::vector<Element *>::iterator aElementIt = LocateElement(id);
	if(  aElementIt != _element_storage.end())
		Delete(aElementIt);
}

// return element present at row,col if it belongs to playercontainer
Element * PlayerContainer::GetElement(int row, int col)
{
	Element * aElement = (Element *)_objectmap_pointer->GetObject(row, col);
	return GetElement(aElement->GetCurStat(STAT_ID));
}

// return element with ID only own elements are checked
Element * PlayerContainer::GetElement(int id)
{
	std::vector<Element *>::iterator aElementIt = LocateElement(id);
	if(  aElementIt != _element_storage.end())
		return((*aElementIt));	
	return NULL;
}

// Returns an element and removes it from own list
Element * PlayerContainer::GiveElement(int row, int col)
{
	Element * element = GetElement(row,col);
	if(element)
		return GiveElement(element->GetCurStat(STAT_ID));	
	return NULL;
}

// Returns an element and removes it from own list
Element * PlayerContainer::GiveElement(int id)
{
	Element * element = GetElement(id);
	if(element)
		_element_storage.erase(LocateElement(id));
	return element;
}

// adds a already created Element to this container
void PlayerContainer::AddExistingElement(Element * element)
{
	if(element)
		_element_storage.push_back(element);	
}

/*---- Private ----*/

// TODO create this method
int PlayerContainer::DirectionTowardsCenter(Element * sel_unit)
{
/*	// relative origin => center of the unit image
	f_origin = iso2scr(sel_unit->GetX() + (sel_unit->GetImage()->GetW() / 2),
								sel_unit->GetY() + (sel_unit->GetImage()->GetH() / 2),
								tm.GetOriginX(),  tm.GetOriginY());

	// get orientation
	orientation = RelDirection((int)round(f_origin->x), (int)round(f_origin->y),
											sel_unit->GetX(), sel_unit->GetY());

	delete f_origin;
	*/
	return SOUTH;
}

// Delete any element type
void PlayerContainer::Delete(std::vector<Element *>::iterator it)
{
	if(it != _element_storage.end())
	{	
		IsoObject * isoobject = (*it);
		Element * element = (*it);
		
		// Remove element from objectmap				
		_objectmap_pointer->RemoveObject(isoobject);
		
		Unit * unit = NULL;
		Building * build = NULL;
		SceneElement * scene = NULL;
		
		// Delete correct element type
		if(element->GetType() == UNIT)
			if((unit = dynamic_cast<Unit *> (element)))
				delete unit;
		else if(element->GetType() == BUILDING)
			if((build = dynamic_cast<Building *> (element)))
				delete build;
		else if(element->GetType() == SCENE_ELEMENT)
			if((scene = dynamic_cast<SceneElement *> (element)))
				delete scene;

		// Delete from container
		_element_storage.erase(it);
	}
}

// Search in storage if element with same id is there
std::vector<Element *>::iterator PlayerContainer::LocateElement(int id)
{
	std::vector<Element *>::iterator it = _element_storage.begin();
	
	for(; it != _element_storage.end(); it++)
	{
		if( (*it)->GetCurStat(STAT_ID) == id)
			return it;
	}
	// it == _element_storage.end()
	return it;
}

PlayerContainer::~PlayerContainer()
{
	int size = _element_storage.size();
	std::vector<Element *>::iterator it;

	// delete all Elements from the vector
	for( int i = 0; i < size; i++ )
	{
		it = _element_storage.begin();
		Delete(it);
	}
}
