#include "XmlLoader.h"

using namespace std;
using namespace graphic;

XmlLoader::XmlLoader(const char *xmlpath)
{
	_xmldoc = new TiXmlDocument(xmlpath);

	///	Load test
	if (!_xmldoc->LoadFile())
		throw Exception("XmlLoader", _xmldoc->ErrorDesc());

	ResetCache();
	
	// read the first child
	_tag = _xmldoc->FirstChildElement();

	if(_tag == NULL)
		throw Exception("XmlLoader", "FirstChildElement() returned NULL, empty file?");

	// set the Element type
	if(!strcmp(_tag->Value(), "unit"))
		_type = UNIT;
	else if(!strcmp(_tag->Value(), "building"))
		_type = BUILDING;
	else if(!strcmp(_tag->Value(), "sceneelement"))
		_type = SCENE_ELEMENT;
	else if(!strcmp(_tag->Value(), "terrainelement"))
			_type = TERRAIN_ELEMENT;

	LoadCache();
}

XmlLoader::XmlLoader(TiXmlDocument * xmldoc)
{
	_xmldoc = xmldoc;
	
		///	Load test
	if ( !_xmldoc->LoadFile())
	{
		printf("XmlLoader::XmlLoader(TiXmlDocument * xmldoc) Error='%s'. \n", _xmldoc->ErrorDesc());
	}
	ResetCache();
	
	/// Is it a Unit document
	if ((_tag = _xmldoc->FirstChildElement("unit")))
		_type = UNIT;
	/// Is it a Building document	
	else if((_tag = _xmldoc->FirstChildElement("building")))
		_type = BUILDING;
	/// Is it a scene element document	
	else if((_tag = _xmldoc->FirstChildElement("sceneelement")))
		_type = SCENE_ELEMENT;
	/// Is it a terrainelement document	
	else if((_tag = _xmldoc->FirstChildElement("terrainelement")))
		_type = TERRAIN_ELEMENT;
		
	LoadCache();
}

XmlLoader::~XmlLoader()
{ 
 	ResetCache(); 
 	_xmldoc->Clear();
 	delete _xmldoc;
}

// Jump to the next unit/building/element tag
bool XmlLoader::NextElement()
{
	_tag = _tag->NextSiblingElement();

	if(_tag)
	{
		ResetCache();
		if(!strcmp(_tag->Value(), "unit"))
		{
			_type = UNIT;
			LoadCache();
			return true;
		}
		else if(!strcmp(_tag->Value(), "building"))
		{
			_type = BUILDING;
			LoadCache();
			return true;
		}
		else if(!strcmp(_tag->Value(), "sceneelement"))
		{
			_type = SCENE_ELEMENT;
			LoadCache();
			return true;
		}
		else if(!strcmp(_tag->Value(), "terrainelement"))
		{
			_type = TERRAIN_ELEMENT;
			LoadCache();
			return true;
		}
	}

	return false;
}

void XmlLoader::LoadCache()
{
	TiXmlElement *el = 0;

	// Look at all elements inside the tag
	for(el = _tag->FirstChildElement();	 el; el = el->NextSiblingElement())
	{
		if(_type == UNIT)
		{					
			if(!strcmp(el->Value(), "chassis"))
			{
				if(el->FirstChildElement())	//Xml data is in the same file
					LoadXmlData(el);
				else							//xml data is in a new file
					LoadXmlData(el->GetText(), "chassis");
			}			
			else if(!strcmp(el->Value(), "pilot"))
			{
				if(el->FirstChildElement())	//Xml data is in the same file
					LoadXmlData(el);
				else							//xml data is in a new file
					LoadXmlData(el->GetText(), "pilot");	
			}										
			else if(!strcmp(el->Value(), "weapon"))
			{
				if(el->FirstChildElement())	//Xml data is in the same file
					LoadXmlData(el);
				else							//xml data is in a new file
					LoadXmlData(el->GetText(), "weapon");
			}										
			else if(!strcmp(el->Value(), "equipment"))
			{
				if(el->FirstChildElement())	//Xml data is in the same file
					LoadXmlData(el);
				else							//xml data is in a new file
					LoadXmlData(el->GetText(), "equipment");
			}				
			else
			{ 
				// statistical values check
				IdentifyStat(el);
				// basic values check;
				IdentifyBasic(el);
			}
		}
		else
		{
			IdentifyStat(el);
			IdentifyBasic(el);
		}
	}	
}

// Adds ID number to statistical values and return all stats
Statsmap XmlLoader::GetStats(int id)
{
	StatisticalValue * stat = NULL;
	_stats.insert(make_pair(STAT_ID, stat = new StatisticalValue(id)));
	return _stats;	
}

/* ------- Private -------*/

void XmlLoader::IdentifyStat(TiXmlElement *tag)
{
	StatisticalValue * stat = NULL;
	pair< Statsmap::iterator, bool > sucess = make_pair(_stats.begin(), true);
	
	// Stats are deleted in its target element destructor
	// insert value into the map with #define as key
	if(!strcmp(tag->Value(), "hitpoints"))
		sucess = _stats.insert(make_pair(STAT_ENERGY, stat = new StatisticalValue(atoi(tag->GetText()))));
		
	else if(!strcmp(tag->Value(), "vision"))	
		sucess = _stats.insert(make_pair(STAT_VISIBILITY, stat = new StatisticalValue(atoi(tag->GetText()))));
		
	else if(!strcmp(tag->Value(), "actionpoints"))	
		sucess = _stats.insert(make_pair(STAT_ACTION, stat = new StatisticalValue(atoi(tag->GetText()))));

	 else if(!strcmp(tag->Value(), "experince"))	
	 		sucess = _stats.insert(make_pair(STAT_EXPERIENCE, stat = new StatisticalValue(atoi(tag->GetText()))));

	 else if(!strcmp(tag->Value(), "movementcost"))	
	 		sucess = _stats.insert(make_pair(STAT_MOVEMENT_COST, stat = new StatisticalValue(atoi(tag->GetText()))));
	
	/// --DUMMY-- replace #xxX*Xxx# with Xml tag and #define from StatisticalValue.h
	/// else if(!strcmp(tag->Value(), "#xxX*Xxx#"))	
	/// 		sucess = _stats.insert(make_pair(#xxX*Xxx#, stat = new StatisticalValue(atoi(tag->GetText()))));
	
	//If any insert failed then the value already exist, add the value to existing instead
	if(!sucess.second)
	{
		//new stat is not needed old already exist, delete it and add value to old
		delete stat;
		// *sucess.first == map pair , map pair second == StatisticalValue *
		((*sucess.first).second)->AddMax(atoi(tag->GetText()));
		((*sucess.first).second)->RestoreCur();
	}

}

void XmlLoader::LoadImages(const char *path)
{
	Image * img;
	string string_path = path;

	//FIXME temporary solution
		
	// Images are deleted in there target element destructor
	try
	{
		img = gtracker->GetImage(string_path + "01.png", screen->GetGraphicMode());
		_views.push_back(img);
		img = gtracker->GetImage(string_path + "02.png", screen->GetGraphicMode());
		_views.push_back(img);
		// If element has more then 2 images, Unit
		if(ImageFileExists(string_path + "03.png"))
		{
			img = gtracker->GetImage(string_path + "03.png", screen->GetGraphicMode());
			_views.push_back(img);
			img = gtracker->GetImage(string_path + "04.png", screen->GetGraphicMode());
			_views.push_back(img);
			img = gtracker->GetImage(string_path + "05.png", screen->GetGraphicMode());
			_views.push_back(img);
			img = gtracker->GetImage(string_path + "06.png", screen->GetGraphicMode());
			_views.push_back(img);
			img = gtracker->GetImage(string_path + "07.png", screen->GetGraphicMode());
			_views.push_back(img);
			img = gtracker->GetImage(string_path + "08.png", screen->GetGraphicMode());
			_views.push_back(img);
		}
	}
	catch(Exception e)
	{
		e.PrintError();
		exit(-1);
	}	
}

void XmlLoader::IdentifyBasic(TiXmlElement *tag)
{
	if(!strcmp(tag->Value(), "type"))
		_type = atoi(tag->GetText());
	else if(!strcmp(tag->Value(), "direction"))
		_direction = IdDirection(tag->GetText());
	else if(!strcmp(tag->Value(), "faction"))
		_faction = IdFaction(tag->GetText());
	else if(!strcmp(tag->Value(), "imagepath"))
		LoadImages(tag->GetText());
	else if(!strcmp(tag->Value(), "rowpos"))
		_rowpos = atoi(tag->GetText());
	else if(!strcmp(tag->Value(), "rowsize"))
		_rowsize = atoi(tag->GetText());
	else if(!strcmp(tag->Value(), "colpos"))
		_colpos = atoi(tag->GetText());
	else if(!strcmp(tag->Value(), "colsize"))
		_colsize = atoi(tag->GetText());
}

bool XmlLoader::LoadXmlData(const char *xmlpath, const char *type)
{
	TiXmlDocument xmldoc(xmlpath);

	///	Load test, fix error control
	if ( !xmldoc.LoadFile())
	{
		printf("XmlLoader::LoadXmlData(char *xmlpath, char *type) Error='%s'. Type='%s'. Path='%s'. Exiting.\n", xmldoc.ErrorDesc(), type, xmlpath);
		return false;
	}	

	TiXmlElement *type_tag = 0;
	type_tag = xmldoc.FirstChildElement(type);
	
	//tag exists
	if (type_tag)
	{
		TiXmlElement *el = 0;
		//TiXmlAttribute *atr = 0;

		// Look at all elements inside the unit tag
		for(el = type_tag->FirstChildElement();	 el; el = el->NextSiblingElement())
		{
		//	atr = el->FirstAttribute();
		//	if(atr)
		//		cout << atr->Value() << endl;				
			IdentifyStat(el);
		}
	}
	
	/// Delete our allocated document and return success
	xmldoc.Clear();
	return true;
}

bool XmlLoader::LoadXmlData(TiXmlElement *tag)
{
	TiXmlElement *el = 0;

	// Look at all elements inside the unit tag
	for(el = tag->FirstChildElement();	 el; el = el->NextSiblingElement())
	{			
		IdentifyStat(el);
	}
	
	return true;
}

int XmlLoader::IdDirection(const char *direction)
{	
	if(!strcmp(direction, "north")) 
		return NORTH;
	if(!strcmp(direction, "north_east")) 
		return NORTH_EAST;
	if(!strcmp(direction, "east")) 
		return EAST;
	if(!strcmp(direction, "south_east")) 
		return SOUTH_EAST;
	if(!strcmp(direction, "south")) 
		return SOUTH;
	if(!strcmp(direction, "south_west")) 
		return SOUTH_WEST;
	if(!strcmp(direction, "west")) 
		return WEST;
	if(!strcmp(direction, "north_west")) 
		return NORTH_WEST;		
		
	return SOUTH;	
}

int XmlLoader::IdFaction(const char *faction)
{	
	if(!strcmp(faction, "no_faction")) 
		return F_NO_FACTION;
	if(!strcmp(faction, "mercenaries")) 
		return F_MERCENARIES;
	if(!strcmp(faction, "ngg")) 
		return F_NGG;
	if(!strcmp(faction, "swa")) 
		return F_SWA;	
		
	return F_NO_FACTION ;	
}

void XmlLoader::ResetCache()
{
	_stats.clear();
	_views.clear();
	_type = SCENE_ELEMENT;
	_direction = SOUTH;
	_faction = F_NO_FACTION;
	_rowpos = 1;
	_rowsize = 1;
	_colpos = 1;
	_colsize = 1;	
}
