#ifndef XMLLOADER_H_
#define XMLLOADER_H_

#include <vector>
#include <map>
#include <utility>

#include "graphic/Image.h"
#include "graphic/GraphicTracker.h"

#include "tinyxml/tinyxml.h"

#include "Exception.h"
#include "Unit.h"
#include "Building.h"
#include "IsoObjectsMap.h"

#define DEFAULT_INT_RETURN 1
#define DEFAULT_STRING_RETURN "1"

/// Loads data from XML files
class XmlLoader
{
	private:
	// The loaded Xml data
	TiXmlDocument * _xmldoc;
	// First child tag of the document
	TiXmlElement * _tag;
	
	// Map containing pointers to several statisticalvalues, later copied to an element
	Statsmap _stats;
	// <imagepath>, vector, later copied to an element
	std::vector < graphic::Image * > _views;
	// <type>, The type the current xml block is point at, default SCENE_ELEMENT
	int _type;
	// <direction>, default SOUTH
	int _direction;
	// <faction>, default F_NO_FACTION
	int _faction;
	// <rowpos>, element row position, default 1
	int _rowpos;
	// <rowsize>, element row size on the map, default 1
	int _rowsize;
	// <colpos>, element colon position, default 1
	int _colpos;
	// <colsize>, element colon size on the map, default 1
	int _colsize;
	
	/// Populates the cache with values from the xml file
	void LoadCache();
	/// Load images into the cache
	void LoadImages(const char *path);
	/// Identifies stat tags and adds there value into the stats map
	void IdentifyStat(TiXmlElement *tag);
	/// Identifies and stores basic element info into cache
	void IdentifyBasic(TiXmlElement *tag);
	/// loads type from next xml file
	bool LoadXmlData(const char *xmlpath, const char *type);
	/// loads type from same xml file
	bool LoadXmlData(TiXmlElement *tag);
	/// Return a direction according to tag
	int IdDirection(const char *direction);
	/// Return faction
	int IdFaction(const char *faction);
	
	public:
	/// String constructor
	XmlLoader(const char *xmlpath);
	/// Xml doc constructor
	XmlLoader(TiXmlDocument * xmldoc);
	/// Destructor
	virtual ~XmlLoader ();
	
	/// Jump to the next unit/building/element tag
	bool NextElement();
	/// Resets cache values to default
	void ResetCache();
	
	// Get methods
	std::vector < graphic::Image * > GetImages() {return _views;};
	Statsmap GetStats(int id);
	int GetDirection() {return _direction;};
	int GetFaction() {return _faction;};
	int GetRowPos() {return _rowpos;};
	int GetRowSize() {return _rowsize;};
	int GetColPos() {return _colpos;};
	int GetColSize() {return _colsize;};
	int GetType() {return _type;};
};

#endif /*XMLLOADER_H_*/
